module Vulgata
  module Strategies
    class Globalize

      # saves a single translation
      def self.save_translation translation_state
        resource = translation_state.item
        ::Globalize.with_locale(translation_state.locale.to_sym) do
          resource.attributes = translation_state.translation_data
          resource.save(validate: false)
        end
      end

      def self.scope_by_source_items items
        items
      end

      # returns the source translation data hash with locale key-value pair
      def self.source_translation instance
        instance.translations.order(created_at: :asc, id: :asc).first
      end

      # returns the translation data along with some meta data (locale, timestamps)
      def self.translation_data_with_meta translation_state
        translation_state.item.translations.find_by(locale: translation_state.locale)
      end

      # returns the translation data in a form of { attr1_name: attr1_value, attr2_name: attr2_value }
      def self.translation_data translation_state
        if translation_data_with_meta translation_state
          ::Globalize.with_locale(translation_state.locale.to_sym) do
            return translation_state.item.translated_attributes.with_indifferent_access
          end
        end

        nil
      end

      # returns array of translated attribute name (i.e [:title, :body])
      def self.translated_attribute_names klass
        klass.translated_attribute_names
      end

      # query the translation, used when searching
      # the query_string is not used in the first line as a raw string for security reasons (sql injections)
      # the collection might be n filtered query on the class (for already filtered queries)
      def self.where_like_query klass, query, query_params,  collection
        query = query % { table_name: klass.translation_class.table_name }
        (collection ||= klass).joins(:translations).where(query, query_params)
      end

      # checks if a translated attribute is changed
      def self.translation_data_changed? instance
        trans_attr_names = instance.translated_attribute_names.map(&:to_s)
        instance.changed.select{ |attr| trans_attr_names.include?(attr) }.any?
      end
    end
  end
end
