module Vulgata
  module Strategies
    class LocaleColumnTranslations

      def initialize(translation_identifier_attribute, translated_attributes)
        @translation_identifier_attribute = translation_identifier_attribute
        @translated_attributes = translated_attributes
      end

      # saves a single translation
      def save_translation translation_state
        resource = translation_state.source_translation_state.item_type.constantize.find_or_initialize_by(:locale => translation_state.locale, @translation_identifier_attribute => translation_state.item[@translation_identifier_attribute])
        resource.vlg_skip_init = true
        resource.vlg_skip_pendify = true
        resource.attributes = translation_state.translation_data
        resource.save(validate: false)
      end

      # when translations are stored in the same table the sources are distincted by the locale
      def scope_by_source_items items
        items.where(locale: I18n.default_locale)
      end

      # returns the source translation data hash with locale key-value pair
      def source_translation instance
        instance.class.find_by(:locale => I18n.default_locale, @translation_identifier_attribute => instance[@translation_identifier_attribute])
      end

      # returns the translation data along with some meta data (locale, timestamps)
      def translation_data_with_meta translation_state
        translation_state.item_class.find_by(:locale => translation_state.locale, @translation_identifier_attribute => translation_state.item[@translation_identifier_attribute])
      end

      # returns the translation data in a form of { attr1_name: attr1_value, attr2_name: attr2_value }
      def translation_data translation_state
        if data = translation_data_with_meta(translation_state)
          return data.slice(*@translated_attributes).with_indifferent_access
        end
        nil
      end

      # returns array of translated attribute name (i.e [:title, :body])
      def translated_attribute_names klass
        @translated_attributes
      end

      def where_like_query klass, query, query_params, collection
        query = query % { table_name: klass.table_name }
        (collection ||= klass).where(query, query_params)
      end

      # checks if a translated attribute is changed
      def translation_data_changed? instance
        translated_attributes = @translated_attributes.map(&:to_s)
        instance.changed.select{ |attr| translated_attributes.include?(attr) }.any?
      end
    end
  end
end
